@@ -8,7 +8,6 @@ This module contains the Authorization object.
 
 from github3.decorators import requires_basic_auth
 from github3.models import GitHubCore
-from json import dumps
 
 
 class Authorization(GitHubCore):
@@ -64,24 +63,22 @@ class Authorization(GitHubCore):
         :returns: bool
         """
         success = False
+        json = None
         if scopes:
             d = {'scopes': scopes}
-            json = self._json(self._post(self._api, data=dumps(d)), 200)
-            self._update_(json)
-            success = True
+            json = self._json(self._post(self._api, data=d), 200)
         if add_scopes:
             d = {'add_scopes': add_scopes}
-            json = self._json(self._post(self._api, data=dumps(d)), 200)
-            self._update_(json)
-            success = True
+            json = self._json(self._post(self._api, data=d), 200)
         if rm_scopes:
             d = {'remove_scopes': rm_scopes}
-            json = self._json(self._post(self._api, data=dumps(d)), 200)
-            self._update_(json)
-            success = True
+            json = self._json(self._post(self._api, data=d), 200)
         if note or note_url:
             d = {'note': note, 'note_url': note_url}
-            json = self._json(self._post(self._api, data=dumps(d)), 200)
+            json = self._json(self._post(self._api, data=d), 200)
+
+        if json:
             self._update_(json)
             success = True
+
         return success
